﻿using System;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 반복 작업자
    /// </summary>
    public class RepeatWorker : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 태스크 헬퍼
        /// </summary>
        private TaskHelper taskHelper = null;

        /// <summary>
        /// 루프 계속 여부
        /// </summary>
        private bool continueLoop = true;

        /// <summary>
        /// 휴지 여부
        /// </summary>
        private bool isSleep = false;

        /// <summary>
        /// 휴지 시간
        /// </summary>
        private int sleepTime;

        /// <summary>
        /// 작업 액션
        /// </summary>
        private Action<object> workAction;

        /// <summary>
        /// 작업 매개 변수
        /// </summary>
        private object workParameter;

        /// <summary>
        /// 실행 여부
        /// </summary>
        private bool isRunning = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 휴지 시간 - SleepTime

        /// <summary>
        /// 휴지 시간
        /// </summary>
        public int SleepTime
        {
            get
            {
                return this.sleepTime;
            }
            set
            {
                this.sleepTime = value;
            }
        }

        #endregion
        #region 작업 액션 - WorkAction

        /// <summary>
        /// 작업 액션
        /// </summary>
        public Action<object> WorkAction
        {
            get
            {
                return this.workAction;
            }
            set
            {
                this.workAction = value;
            }
        }

        #endregion
        #region 작업 매개 변수 - WorkParameter

        /// <summary>
        /// 작업 매개 변수
        /// </summary>
        public object WorkParameter
        {
            get
            {
                return this.workParameter;
            }
            set
            {
                this.workParameter = value;
            }
        }

        #endregion
        #region 실행 여부 - IsRunning

        /// <summary>
        /// 실행 여부
        /// </summary>
        public bool IsRunning
        {
            get
            {
                return this.isRunning;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RepeatWorker()

        /// <summary>
        /// 생성자
        /// </summary>
        public RepeatWorker()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작하기 - Start()

        /// <summary>
        /// 시작하기
        /// </summary>
        public void Start()
        {
            try
            {
                if(this.isRunning)
                {
                    return;
                }

                this.isRunning = true;

                if(this.taskHelper != null)
                {
                    if(this.taskHelper.IsRunning)
                    {
                        this.taskHelper.Abort();
                    }

                    this.taskHelper = null;
                }

                this.taskHelper = new TaskHelper();

                this.taskHelper.Start(ExecuteWorkAction, null);
            }
            catch(Exception exception)
            {
                this.isRunning = false;

                throw exception;
            }
        }

        #endregion
        #region 중단하기 - Stop()

        /// <summary>
        /// 중단하기
        /// </summary>
        public async void Stop()
        {
            try
            {
                if(!this.isRunning)
                {
                    return;
                }

                this.isRunning = false;

                this.continueLoop = false;

                await Task.Delay(500);

                if(this.taskHelper != null && this.taskHelper.IsRunning)
                {
                    if(this.isSleep)
                    {
                        this.taskHelper.Abort();
                    }
                }
            }
            catch(Exception)
            {
            }
        }

        #endregion

        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            Stop();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 작업 액션 실행하기 - ExecuteWorkAction(parameter)

        /// <summary>
        /// 작업 액션 실행하기
        /// </summary>
        /// <param name="parameter">매개 변수</param>
        private async void ExecuteWorkAction(object parameter)
        {
            while(this.continueLoop)
            {
                try
                {
                    this.workAction?.Invoke(this.workParameter);
                }
                catch(Exception)
                {
                }

                if(!this.continueLoop)
                {
                    break;
                }

                this.isSleep = true;

                await Task.Delay(this.sleepTime);

                this.isSleep = false;
            }
        }

        #endregion
    }
}