﻿using System;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 태스크 헬퍼
    /// </summary>
    public class TaskHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 실행 여부
        /// </summary>
        private bool isRunning = false;

        /// <summary>
        /// 취소 토큰 소스
        /// </summary>
        private CancellationTokenSource cancellationTokenSource = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 실행 여부 - IsRunning

        /// <summary>
        /// 실행 여부
        /// </summary>
        public bool IsRunning
        {
            get
            {
                return this.isRunning;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작하기 - Start(workAction, workParameter)

        /// <summary>
        /// 시작하기
        /// </summary>
        /// <param name="workAction">작업 액션</param>
        /// <param name="workParameter">작업 매개 변수</param>
        public void Start(Action<object> workAction, object workParameter)
        {
            this.isRunning = true;

            this.cancellationTokenSource = new CancellationTokenSource();

            Task.Factory.StartNew
            (
                () =>
                {
                    try
                    {
                        using(this.cancellationTokenSource.Token.Register(Thread.CurrentThread.Abort))
                        {
                            workAction(workParameter);
                        }
                    }
                    catch(ThreadAbortException)
                    {
                        this.isRunning = false;
                    }
                },
                this.cancellationTokenSource.Token
            );
        }

        #endregion
        #region 중단하기 - Abort()

        /// <summary>
        /// 중단하기
        /// </summary>
        public void Abort()
        {
            this.cancellationTokenSource.Cancel();
        }

        #endregion
    }
}